/*- - - - - - - - - - - - - - - - - - - - - - - - - - - -
	CALEDIT.CPP

	TCalendarEdit Class

	Author:     Daniel BERMAN (CompuServe: 100567,3464)
	Date:       09 jully 1995 (1.0)
	Date:       13 jully 1995 (1.01)

*/

#include <owl\owlpch.h>
#pragma hdrstop

#include <stdlib.h>

#include "intldate.h"
#include "caledit.h"
#include "caledit.rh"


static TIntlDate::HowToPrint PrintOption = TIntlDate::Numbers;


class TPopupCalWin : public TWindow
{
public:

	TPopupCalWin(TWindow* parent, TCalendarEdit* edit, TModule* module = 0);
	~TPopupCalWin();

	const TIntlDate& GetDate();

	void SetupWindow();

	void SetText(TIntlDate& date);

private:

	TFont* Font;
	int FontWidth;
	int FontHeight;

	TIntlDate Date;

	int MonthRow;
	int WeekDaysRow;
	int DaysRow;
	int StartCol;

	bool Dragging;
	TRect CurRect;

	TBitmap* BmpPrev;
	TBitmap* BmpNext;
	TRect BtnPrevRect;
	TRect BtnNextRect;

	TCalendarEdit* CalEdit;

	void DrawFocusRect(TDC& dc, int x, int y);
	void Paint(TDC& dc, bool erase, TRect& rect);
	void PaintBorders(TDC& dc, TRect& rect);

	void EvKillFocus(HWND hwndGetFocus);
	void EvLButtonDown(uint modKeys, TPoint& point);
	void EvLButtonUp(uint modKeys, TPoint& point);
	void EvMouseMove(uint modKeys, TPoint& point);

	DECLARE_RESPONSE_TABLE(TPopupCalWin);
};

class TCalendarBtn : public TButton
{
public:

	TCalendarBtn(TWindow *parent, TCalendarEdit* edit, TModule* module);
	~TCalendarBtn();

	char far* 			GetClassName();
	void					SetupWindow();

	const TIntlDate&	GetDate();

private:

	TCalendarEdit* CalEdit;
	TPopupCalWin* CalWin;
	TBitmap* BmpUp;
	TBitmap* BmpDn;

	void EvClicked();
	void EvDrawItem(uint ctrlId, DRAWITEMSTRUCT far& drawInfo);

	DECLARE_RESPONSE_TABLE(TCalendarBtn);
};


//------------------------------------------------------------------------------
// TCalendarBtn
//------------------------------------------------------------------------------

DEFINE_RESPONSE_TABLE1(TCalendarBtn, TButton)
	EV_WM_DRAWITEM,
	EV_NOTIFY_AT_CHILD(BN_CLICKED, EvClicked),
END_RESPONSE_TABLE;

TCalendarBtn::TCalendarBtn(TWindow *parent, TCalendarEdit* edit, TModule* module)
	: TButton(parent, 32489, "", 0, 0, 0, 0, false, module)
{
	Attr.Style |= BS_OWNERDRAW;
	Attr.Style &= ~WS_TABSTOP;

	CalEdit = edit;
	CalWin = new TPopupCalWin(Parent, edit, module);
   CalWin->DisableAutoCreate();

	BmpUp = new TBitmap(*GetApplication(), IDB_CALBTNUP);
	BmpDn = new TBitmap(*GetApplication(), IDB_CALBTNDN);
}

TCalendarBtn::~TCalendarBtn()
{
	delete BmpUp;
	delete BmpDn;
}

void TCalendarBtn::EvClicked()
{
	if (*CalWin)
		return;

	TRect r = CalEdit->GetWindowRect();
	CalWin->Attr.X = r.left;
	CalWin->Attr.Y = r.bottom;
	CalWin->Create();
}

void TCalendarBtn::EvDrawItem(uint /*ctrlId*/, DRAWITEMSTRUCT far& drawInfo)
{
	TDC dc(drawInfo.hDC);
	TMemoryDC memDC(dc);

	if (drawInfo.itemState & ODS_SELECTED)
		memDC.SelectObject(*BmpDn);
	else
		memDC.SelectObject(*BmpUp);
	dc.BitBlt(0, 0, Attr.W, Attr.H, memDC, 0, 0);
}

char far* TCalendarBtn::GetClassName()
{
	return "BUTTON";
}

const TIntlDate& TCalendarBtn::GetDate()
{
	return CalWin->GetDate();
}

void TCalendarBtn::SetupWindow()
{
	TButton::SetupWindow();

	TRect	rcEdit = CalEdit->GetClientRect();
	TRect	rcThis = rcEdit;

	rcThis.left	= rcThis.right - BmpUp->Width();
	rcEdit.right = rcThis.left;

	CalEdit->MapWindowPoints(*Parent, rcThis, 2);
	rcThis = rcThis.InflatedBy(0, 1);

	SetWindowPos(*CalEdit, rcThis, SWP_NOACTIVATE);
	CalEdit->SetWindowPos(NULL, 0, 0, rcEdit.right, rcEdit.bottom,
	                      SWP_NOMOVE | SWP_NOACTIVATE | SWP_NOZORDER);
}


//------------------------------------------------------------------------------
// TPopupCalWin
//------------------------------------------------------------------------------

DEFINE_RESPONSE_TABLE1(TPopupCalWin, TWindow)
	EV_WM_KILLFOCUS,
	EV_WM_LBUTTONDOWN,
	EV_WM_LBUTTONUP,
	EV_WM_MOUSEMOVE,
END_RESPONSE_TABLE;

TPopupCalWin::TPopupCalWin(TWindow* parent, TCalendarEdit* edit, TModule* module)
	: TWindow(parent, 0, module)
{
	Attr.Style = WS_POPUPWINDOW | WS_VISIBLE;

	CalEdit = edit;

	Font = new TFont("MS Sans Serif", -8);
	{
		TRect rc;
		TScreenDC dc;
		dc.SelectObject(*Font);
		dc.DrawText("Abc", -1, rc, DT_CENTER | DT_CALCRECT);
		FontWidth = rc.Width();
		FontHeight = rc.Height();
	}

	MonthRow = 0;
	WeekDaysRow = MonthRow + FontHeight;
	DaysRow = WeekDaysRow + FontHeight + 4;
	StartCol = 4;

	Attr.W = (7 * FontWidth) + ( 2 * StartCol);
	Attr.H = DaysRow + (6 * FontHeight) + 4;

	Dragging = false;

	BmpPrev = new TBitmap(*GetApplication(), IDB_CALBTNPREV);
	BtnPrevRect = TRect(2, 2, BmpPrev->Width() + 2, BmpPrev->Height() + 2);

	BmpNext = new TBitmap(*GetApplication(), IDB_CALBTNNEXT);
	BtnNextRect.left = Attr.W - BmpNext->Width() - 4;
	BtnNextRect.top = 2;
	BtnNextRect.right = BtnNextRect.left + BmpNext->Width();
	BtnNextRect.bottom = BtnNextRect.top + BmpNext->Height();

	SetBkgndColor(TColor::LtGray);
}

TPopupCalWin::~TPopupCalWin()
{
	delete Font;
	delete BmpPrev;
	delete BmpNext;
}

void TPopupCalWin::DrawFocusRect(TDC& dc, int x, int y)
{
	TRect rect = TRect(StartCol + x * FontWidth,
							 DaysRow + y * FontHeight,
							 StartCol + x * FontWidth + FontWidth,
							 DaysRow + y * FontHeight + FontHeight);

	TIntlDate date(((y+1)*7) - (7-(x+1)) - (Date.WeekDay(1)-1), Date.Month(), Date.Year());
	if (!date.IsValid())
		return;

	SetText(date);

	dc.SaveDC();

	if (!CurRect.IsEmpty()) {
		dc.SelectObject(TBrush(TColor(TColor::LtGray)));
		dc.PatBlt(CurRect.left, CurRect.top, CurRect.Width() - 1, 1, PATCOPY);
		dc.PatBlt(CurRect.left, CurRect.top, 1, CurRect.Height() - 1, PATCOPY);
		dc.PatBlt(CurRect.right - 1, CurRect.top, 1, CurRect.Height(), PATCOPY);
		dc.PatBlt(CurRect.left, CurRect.bottom - 1, CurRect.Width(), 1, PATCOPY);
	}

	dc.SelectObject(TBrush(TColor(TColor::Gray)));
	dc.PatBlt(rect.left, rect.top, rect.Width() - 1, 1, PATCOPY);
	dc.PatBlt(rect.left, rect.top, 1, rect.Height() - 1, PATCOPY);
	dc.SelectObject(TBrush(TColor(TColor::White)));
	dc.PatBlt(rect.right - 1, rect.top, 1, rect.Height(), PATCOPY);
	dc.PatBlt(rect.left, rect.bottom - 1, rect.Width(), 1, PATCOPY);

	CurRect = rect;

	dc.RestoreDC();
}

void TPopupCalWin::EvKillFocus(HWND /*hwndGetFocus*/)
{
	CloseWindow();
}

void TPopupCalWin::EvLButtonDown(uint modKeys, TPoint& point)
{
	SetCapture();

	if (BtnPrevRect.Contains(point) || BtnNextRect.Contains(point))
   	return;

	Dragging = true;
	EvMouseMove(modKeys, point);
}

void TPopupCalWin::EvLButtonUp(uint /*modKeys*/, TPoint& point)
{
	ReleaseCapture();

	if (Dragging) {
		Dragging = false;

		int x = ((point.x  - StartCol) / FontWidth) + 1;
		int y = ((point.y - DaysRow) / FontHeight) + 1;
		TIntlDate date((y*7) - (7-x) - (Date.WeekDay(1)-1), Date.Month(), Date.Year());
		if (!date.IsValid())
			return;

		SetText(date);
		CalEdit->SetFocus();

		CloseWindow();
	}
	else if (BtnPrevRect.Contains(point)) {
		int days = Date.DaysInMonth(Date.Month()-1 < 1 ? 12 : Date.Month()-1);
		if (days < Date.DayOfMonth())
      	Date -= Date.DayOfMonth();
		else
			Date -= days;
		Invalidate();
	}
	else if (BtnNextRect.Contains(point)) {
		Date += Date.DaysInMonth();
		Invalidate();
	}
}

void TPopupCalWin::EvMouseMove(uint modKeys, TPoint& point)
{
	if (Dragging)
		if (TRect(StartCol, DaysRow, StartCol + 7 * FontWidth,
					 DaysRow + 6 * FontHeight).Contains(point)) {
			TClientDC dc(*this);
			DrawFocusRect(dc,
							  (point.x - StartCol) / FontWidth,
							  (point.y - DaysRow) / FontHeight);
		}
	TWindow::EvMouseMove(modKeys, point);
}

const TIntlDate& TPopupCalWin::GetDate()
{
	return Date;
}

void TPopupCalWin::Paint(TDC& dc, bool /*erase*/, TRect& /*rect*/)
{
	int i;
	TRect r;
	int weekday = Date.WeekDay(1);
	TRect clientRect = GetClientRect();

	PaintBorders(dc, clientRect);

	dc.SelectObject(*Font);
	dc.SetBkMode(TRANSPARENT);

	TMemoryDC memDC(dc);
	memDC.SelectObject(*BmpPrev);
	dc.BitBlt(BtnPrevRect, memDC, TPoint(0,0));
	memDC.SelectObject(*BmpNext);
	dc.BitBlt(BtnNextRect, memDC, TPoint(0,0));

	char buf[80];
	wsprintf(buf, "%s %d", Date.NameOfMonth(), Date.Year());
	r = TRect(0, MonthRow, clientRect.right, MonthRow + FontHeight);
	dc.DrawText(buf, -1, r, DT_CENTER | DT_VCENTER);

	r = TRect(StartCol, WeekDaysRow, StartCol + FontWidth, WeekDaysRow + FontHeight);
	for (i = 0; i < 7; i++) {
		dc.DrawText(Date.ShortDayName(i+1).c_str(), -1, r, DT_CENTER | DT_VCENTER);
		r.left += FontWidth;
		r.right += FontWidth;
	}

	dc.SaveDC();
	clientRect.top = WeekDaysRow + FontHeight;
	clientRect.bottom = clientRect.top + 2;
	dc.SelectObject(TBrush(TColor(TColor::Gray)));
	dc.PatBlt(clientRect.left, clientRect.top, clientRect.Width() - 1, 1, PATCOPY);
	dc.SelectObject(TBrush(TColor(TColor::White)));
	dc.PatBlt(clientRect.left, clientRect.bottom - 1, clientRect.Width(), 1, PATCOPY);
	dc.RestoreDC();

	r = TRect(StartCol + ((weekday -1) * FontWidth), DaysRow, StartCol + ((weekday -1) * FontWidth) + FontWidth, DaysRow + FontHeight);
	for (i = 1; i <= Date.DaysInMonth(); i++, weekday++) {
		char buf[3];
		dc.DrawText(itoa(i, buf, 10), -1, r, DT_CENTER | DT_VCENTER);
		r.left += FontWidth;
		r.right += FontWidth;
		if (weekday % 7 == 0) {
			r.left = StartCol;
			r.right = StartCol + FontWidth;
			r.top += FontHeight;
			r.bottom += FontHeight;
		}
	}

	DrawFocusRect(dc, Date.WeekDay()-1, (Date.DayOfMonth()+Date.WeekDay(1)-2)/7);
}

void TPopupCalWin::PaintBorders(TDC& dc, TRect& rect)
{
	dc.SaveDC();
	dc.SelectObject(TBrush(TColor(TColor::White)));
	dc.PatBlt(rect.left, rect.top, rect.Width() - 1, 1, PATCOPY);
	dc.PatBlt(rect.left, rect.top, 1, rect.Height() - 1, PATCOPY);
	dc.SelectObject(TBrush(TColor(TColor::Gray)));
	dc.PatBlt(rect.right - 1, rect.top, 1, rect.Height(), PATCOPY);
	dc.PatBlt(rect.left, rect.bottom - 1, rect.Width(), 1, PATCOPY);
	dc.RestoreDC();
}

void TPopupCalWin::SetText(TIntlDate& date)
{
	TIntlDate::HowToPrint oldPrn = date.SetPrintOption(PrintOption);
	CalEdit->SetText(date.AsString().c_str());
	date.SetPrintOption(oldPrn);
}

void TPopupCalWin::SetupWindow()
{
	TWindow::SetupWindow();

	char str[40];
	CalEdit->GetText(str, sizeof(str));

	TIntlDate::HowToPrint oldPrn = TIntlDate::SetPrintOption(PrintOption);
	Date = TIntlDate(istrstream(str));
	if (!Date.IsValid())
		Date = TIntlDate();
	TIntlDate::SetPrintOption(oldPrn);
}

//------------------------------------------------------------------------------
// TCalendarEdit
//------------------------------------------------------------------------------

TCalendarEdit::TCalendarEdit(TWindow* parent, int id, const char far* text,
									  int x, int y, int w, int h, uint textLen,
									  TModule* module)
	: TEdit(parent, id, text, x, y, w, h, textLen, false, module)
{
	Init();
}

TCalendarEdit::TCalendarEdit(TWindow* parent, int resourceId, uint textLen,
                             TModule* module)
	: TEdit(parent, resourceId, textLen, module)
{
	Init();
}

void TCalendarEdit::Init()
{
	CalBtn = new TCalendarBtn(Parent, this, GetModule());
}

const TIntlDate& TCalendarEdit::GetDate()
{
	return CalBtn->GetDate();
}

TIntlDate::HowToSpeak TCalendarEdit::SetLanguage(TIntlDate::HowToSpeak l)
{
	return TIntlDate::SetLanguageOption(l);
}

TIntlDate::HowToPrint TCalendarEdit::SetPrintOption(TIntlDate::HowToPrint h)
{
	TIntlDate::HowToPrint old = PrintOption;
	PrintOption = h;
	return old;
}



