/*- - - - - - - - - - - - - - - - - - - - - - - - - - - -
	INTLDATE.H

	TIntlDate Class

	Author:     Daniel BERMAN (CompuServe: 100567,3464)
	Date:       13 jully 1995 (1.01)

*/

#ifndef _TintlDate_H
#define _TintlDate_H

#include <cstring.h>
#include <classlib\date.h>
#include <strstream.h>



class TIntlDate
{
public:

	enum HowToPrint {Normal, Terse, Numbers, EuropeanNumbers, European};
	enum HowToSpeak {English = -1, French = 0,};

	TIntlDate();
	TIntlDate(DayTy day, MonthTy month, YearTy year);
	TIntlDate(DayTy day, const char* month, YearTy year);
	TIntlDate(const TDate& date);
	TIntlDate(istream& s);
	~TIntlDate();

	string								AsString() const;

	DayTy									DayOfMonth() const;
	MonthTy								Month() const;
	YearTy								Year() const;
	DayTy									WeekDay() const;
	DayTy									WeekDay(DayTy day) const;

	const char*							NameOfDay() const;
	const char*							NameOfMonth() const;

	int									DaysInMonth() const;
	int									DaysInMonth(MonthTy month) const;

	int									IsValid() const;
	int									Leap()  const;

	static int							AssertIndexOfMonth(MonthTy m);
	static int							AssertWeekDayNumber(DayTy d);

	static const char*				DayName(DayTy weekDayNumber);
	static const string				ShortDayName(DayTy weekDayNumber);
	static const char*				MonthName(MonthTy monthNumber);

	static MonthTy 					IndexOfMonth(const char* nameOfMonth);

	static HowToSpeak					SetLanguageOption(HowToSpeak l);
	static TIntlDate::HowToPrint	SetPrintOption(TIntlDate::HowToPrint h);

	int operator <  (const TIntlDate& date) const;
	int operator <= (const TIntlDate& date) const;
	int operator >  (const TIntlDate& date) const;
	int operator >= (const TIntlDate& date) const;
	int operator == (const TIntlDate& date) const;
	int operator != (const TIntlDate& date) const;

	TIntlDate& const operator = (const TIntlDate& dt);
	TIntlDate& const operator = (const TDate& dt);

	JulTy operator - (const TIntlDate& dt) const;
	friend TIntlDate operator + (const TIntlDate& dt, int dd);
	friend TIntlDate operator + (int dd, const TIntlDate& dt);
	friend TIntlDate operator - (const TIntlDate& dt, int dd);

	void operator ++ ();
	void operator -- ();
	void operator += (int dd);
	void operator -= (int dd);

	friend ostream& operator << (ostream& s, const TIntlDate& d);

private:

	TDate* Date;

	static HowToPrint PrintOption;
	static HowToSpeak Language;

   void EuropeanParseFrom(istream& s);
};


inline int TIntlDate::AssertIndexOfMonth(MonthTy m)
{
	return (m >= 1 && m <= 12);
}

inline int TIntlDate::AssertWeekDayNumber(DayTy d)
{
	return (d >= 1 && d <= 7);
}

// Returns the day of the month of this TIntlDate.
inline DayTy TIntlDate::DayOfMonth() const
{
	return Date->DayOfMonth();
}

// Return the number of days of this TIntlDate.
// Return 0 if "month" is outside of the range 1 through 12, inclusive.
inline int TIntlDate::DaysInMonth() const
{
	return DaysInMonth(Date->Month());
}

inline int TIntlDate::IsValid() const
{
	return Date->IsValid() > 0;
}

inline int TIntlDate::Leap() const
{
	return Date->Leap();
}

// Returns the month of this TIntlDate.
inline MonthTy TIntlDate::Month() const
{
	return Date->Month();
}

// Returns the day's name of this TIntlDate.
inline const char* TIntlDate::NameOfDay() const
{
	return DayName(Date->WeekDay());
}

// Returns the month's name of this TIntlDate.
inline const char* TIntlDate::NameOfMonth() const
{
	return MonthName(Date->Month());
}

// Returns the year of this TIntlDate.
inline YearTy TIntlDate::Year() const
{
	return Date->Year();
}

inline DayTy TIntlDate::WeekDay() const
{
	return Date->WeekDay();
}

inline int TIntlDate::operator < (const TIntlDate& date) const
{
	return (*Date < *date.Date);
}

inline int TIntlDate::operator <= (const TIntlDate& date) const
{
	return (*Date <= *date.Date);
}

inline int TIntlDate::operator > (const TIntlDate& date) const
{
	return (*Date > *date.Date);
}

inline int TIntlDate::operator >= (const TIntlDate& date) const
{
	return (*Date >= *date.Date);
}

inline int TIntlDate::operator == (const TIntlDate& date) const
{
	return (*Date == *date.Date);
}

inline int TIntlDate::operator != (const TIntlDate& date) const
{
	return (*Date != *date.Date);
}

inline JulTy TIntlDate::operator - (const TIntlDate& dt) const
{
	return (*Date - *dt.Date);
}

inline TIntlDate operator + (const TIntlDate& dt, int dd)
{
	return TIntlDate((*(dt.Date) + dd));
}

inline TIntlDate operator + (int dd, const TIntlDate& dt)
{
	return TIntlDate((*(dt.Date) + dd));
}

inline TIntlDate operator - (const TIntlDate& dt, int dd)
{
	return TIntlDate((*(dt.Date) - dd));
}

inline void TIntlDate::operator ++ ()
{
	++(*Date);
}

inline void TIntlDate::operator -- ()
{
	--(*Date);
}

inline void TIntlDate::operator += (int dd)
{
	*Date += dd;
}

inline void TIntlDate::operator -= (int dd)
{
	*Date -= dd;
}


#endif
